""" Core module chemical composition settings
"""
from django.conf import settings

if not settings.configured:
    settings.configure()

AUTO_ESCAPE_XML_ENTITIES = getattr(settings, 'AUTO_ESCAPE_XML_ENTITIES', True)
""" bool: enable or not the auto escape of the XML predefined entities
"""



""" Chemical composition module view
"""
import json

from core_module_chemical_composition_app.api import render_chemical_composition, get_chemical_composition_popup_content
from core_module_chemical_composition_app.settings import AUTO_ESCAPE_XML_ENTITIES
from core_parser_app.tools.modules.views.builtin.popup_module import AbstractPopupModule
from xml_utils.xsd_tree.operations.xml_entities import XmlEntities


class ChemicalCompositionModule(AbstractPopupModule):
    def __init__(self):
        AbstractPopupModule.__init__(self, button_label='Select Elements',
                                     styles=['core_module_periodic_table_app/css/periodic.css',
                                             'core_module_chemical_composition_app/css/'
                                             'chemical_element_composition.css'],
                                     scripts=['core_module_chemical_composition_app/js/events.js',
                                              'core_module_chemical_composition_app/js/'
                                              'chemical_element_composition.js'])

    def _retrieve_data(self, request):
        """ Retrieve module's data

        Args:
            request:

        Returns:

        """
        data = ''
        if request.method == 'GET':
            if 'data' in request.GET:
                data = request.GET['data']
        elif request.method == 'POST':
            if 'elementList' in request.POST:
                element_list = json.loads(request.POST['elementList'])
                if len(element_list) > 0:
                    element_list_xml = ""
                    for element in element_list:
                        element_list_xml += '<constituent>'
                        element_list_xml += "<element>" + element['name'] + "</element>"
                        element_list_xml += "<quantity>" + (XmlEntities().escape_xml_entities(element['qty']) if AUTO_ESCAPE_XML_ENTITIES else element['qty']) + "</quantity>"
                        element_list_xml += "<purity>" + (XmlEntities().escape_xml_entities(element['pur']) if AUTO_ESCAPE_XML_ENTITIES else element['pur']) + "</purity>"
                        element_list_xml += "<error>" + (XmlEntities().escape_xml_entities(element['err']) if AUTO_ESCAPE_XML_ENTITIES else element['err']) + "</error>"
                        element_list_xml += '</constituent>'
                    # set the data
                    data = element_list_xml
        return data

    def _render_data(self, request):
        """ Return module's data rendering

        Args:
            request:

        Returns:

        """
        return render_chemical_composition(self.data, True, True,
                                           'core_module_chemical_composition_app/render_data.html')

    def _get_popup_content(self):
        """ Return module's data rendering
        """
        # rendering data in the edit form
        data_template = render_chemical_composition(self.data, True, True,
                                                    'core_module_chemical_composition_app/edit_data.html')

        return get_chemical_composition_popup_content(self.data, data_template)



""" Url router for the chemical composition module
"""

from django.urls import re_path

from core_module_chemical_composition_app.views import ChemicalCompositionModule

urlpatterns = [
    re_path(r'module-chemical-composition',
            ChemicalCompositionModule.as_view(),
            name='core_module_chemical_composition'),
]



""" Chemical composition API
"""
from django.template import loader

from core_parser_app.tools.modules.views.module import AbstractModule
from xml_utils.xsd_tree.xsd_tree import XSDTree


def get_chemical_composition_popup_content(data, edit_template):
    """ Chemical_composition module's rendering
    """
    # rendering data in the periodic table
    template = AbstractModule.render_template('core_module_periodic_table_app/periodic.html',
                                              {'selected_elements': get_periodic_table_selected_elements(data)})

    # rendering both form in the popup template
    return AbstractModule.render_template('core_module_chemical_composition_app/chemical_composition.html',
                                          {'periodic_table': template,
                                           'data_template': edit_template})


def get_periodic_table_selected_elements(data_constituents):
    """ get selected elements from xml constituents

    Args:
        data_constituents:

    Returns: selected_elements
    """
    selected_elements = []
    constituents = XSDTree.fromstring("<constituents>" + data_constituents + "</constituents>")
    for constituent in constituents:
        constituent_elements = list(constituent)
        for constituent_element in constituent_elements:
            if constituent_element.tag == 'element':
                if constituent_element.text is not None:
                    selected_elements.append(constituent_element.text)
    return selected_elements


def render_chemical_composition(data_constituents, display_purity, display_error, template_url):
    """ render elements from xml constituents using the given template

    Args:
        data_constituents:
        display_purity:
        display_error:
        template_url:

    Returns: selected_elements
    """
    data = []
    if len(data_constituents) > 0:
        constituents = XSDTree.fromstring("<constituents>" + data_constituents + "</constituents>")

        # build data to display
        if len(constituents) > 0:
            for constituent in constituents:
                constituent_elements = list(constituent)
                name = ''
                quantity = ''
                purity = ''
                error = ''
                for constituent_element in constituent_elements:
                    if constituent_element.tag == 'element':
                        if constituent_element.text is not None:
                            name = constituent_element.text
                    elif constituent_element.tag == 'quantity':
                        if constituent_element.text is not None:
                            quantity = constituent_element.text
                    elif constituent_element.tag == 'purity':
                        if constituent_element.text is not None:
                            purity = constituent_element.text
                    elif constituent_element.tag == 'error':
                        if constituent_element.text is not None:
                            error = constituent_element.text

                item = {
                    'name': name,
                    'quantity': quantity,
                    'purity': purity,
                    'error': error
                }

                data.append(item)

    # context building
    context = {
        'purity': display_purity,
        'error': display_error,
        'data': data
    }

    # template loading with context
    template = loader.get_template(template_url)
    return template.render(context)

